/*
3D Visualization Project
MAT 259A - Winter 2021
Author: Lara Floegel-Shetty

  The data visualization from this projects takes the top 2000 non fiction books every year from 2006 to 2019 and partitions them into fourteen segments that
  make up a 3D sphere. Each partition represents a year with all 2000 of the data points from that year placed randomly inside. Then, based on whether they are
  of the same dewey class, points in each of the segments are connected through lines to form constellations. The order in which they are connected is random
  and the constellations formed are differnt each time the data visual is run. All of the constellations of the same dewey class in the different years are of the
  same color to allow for easier comparison and for cohesion.
  
  The user is able to turn off all of the data points to see only the lines or to use the keypad to see certain dewey classes. They can also choose which years they
  want to see using the checkboxes displayed on screen. They can choose whether or not to have rotation and press another key to see the title represented by each data
  point when they hover their mouse over it.
  
Processing Version: 4.0a3
  
Libraries
  controlP5
  peasy
  megamu.mesh: https://leebyron.com/mesh/
  
References
  Guanyu Chen's UI code in the gui file.

*/
import controlP5.*;
import megamu.mesh.*;
import peasy.*;
PeasyCam cam;
ControlP5 cp5;
PFont f;

int n, trans, start, end, p;
float[] randomtrans;
float lat, lon, r, t, d, x, y, z;
Boolean labels,
        rotation,
        data;
Boolean[] off,
          dewey;
color[] coloron;

ArrayList<ArrayList<Book>> book;
ArrayList<PVector[]> coords;
ArrayList<float[][]> points;

Table table2006, table2007, table2008,
      table2009, table2010, table2011,
      table2012, table2013, table2014,
      table2015, table2016, table2017,
      table2018, table2019;
ArrayList<Book> books2006, books2007, books2008,
                books2009, books2010, books2011,
                books2012, books2013, books2014,
                books2015, books2016, books2017,
                books2018, books2019;
void setup() {
  size(1250, 815, P3D);
  cam = new PeasyCam(this, 1000);  
  cam.setMaximumDistance(5000);
  cp5 = new ControlP5(this);
  smooth();
  
  book = new ArrayList<ArrayList<Book>>();
  coords = new ArrayList<PVector[]>();
  points = new ArrayList<float[][]>();
   
  books2006 = new ArrayList<Book>();
  book.add(books2006);
  books2007 = new ArrayList<Book>();
  book.add(books2007);
  books2008 = new ArrayList<Book>();
  book.add(books2008);
  books2009 = new ArrayList<Book>();
  book.add(books2009);
  books2010 = new ArrayList<Book>();
  book.add(books2010);
  books2011 = new ArrayList<Book>();
  book.add(books2011);
  books2012 = new ArrayList<Book>();
  book.add(books2012);
  books2013 = new ArrayList<Book>();
  book.add(books2013);
  books2014 = new ArrayList<Book>();
  book.add(books2014);
  books2015 = new ArrayList<Book>();
  book.add(books2015);
  books2016 = new ArrayList<Book>();
  book.add(books2016);
  books2017 = new ArrayList<Book>();
  book.add(books2017);
  books2018 = new ArrayList<Book>();
  book.add(books2018);
  books2019 = new ArrayList<Book>();
  book.add(books2019);
  
  fillTable(table2006, books2006, "2006top.csv");
  fillTable(table2007, books2007, "2007top.csv");
  fillTable(table2008, books2008, "2008.csv");
  fillTable(table2009, books2009, "2009.csv");
  fillTable(table2010, books2010, "2010.csv");
  fillTable(table2011, books2011, "2011.csv");
  fillTable(table2012, books2012, "2012.csv");
  fillTable(table2013, books2013, "2013.csv");
  fillTable(table2014, books2014, "2014.csv");
  fillTable(table2015, books2015, "2015.csv");
  fillTable(table2016, books2016, "2016.csv");
  fillTable(table2017, books2017, "2017.csv");
  fillTable(table2018, books2018, "2018.csv");
  fillTable(table2019, books2019, "2019.csv");
  
  
  n = 2000;        // number of points
  r = 400;       // sphere's radius
  t = 0;         // rotation accumulator
  p = 361/7;
  trans = 0;
  start = 180 - (361/7);
  end = 180;
  
  off = new Boolean[14];
  coloron = new color[14];
  
  for(int i=0; i<off.length; ++i)
  {
    off[i] = false;
    coloron[i] = color(255);
  }
  
  labels = false;
  rotation = true;
  data = true;
  
  dewey = new Boolean[10];
  
  for(int i=0; i<dewey.length; ++i)
  {
    dewey[i] = false;
  }
  
  for(int i=0; i<14; ++i)
  {
    coords.add(new PVector[n]);
    points.add(new float[n][3]);
  }
  
  randomtrans = new float[n];
  
  drawUpperPoints(points.get(0), randomtrans, start, end, books2006);
  start -= p;
  end -= p;
  
  drawUpperPoints(points.get(1), randomtrans, start, end, books2007);
  start -= p;
  end -= p;
 
  drawUpperPoints(points.get(2), randomtrans, start, end, books2008);
  start -= p;
  end -= p;
  
  drawUpperPoints(points.get(3), randomtrans, start, end, books2009);
  start -= p;
  end -= p;
  
  drawUpperPoints(points.get(4), randomtrans, start, end, books2010);
  start -= p;
  end -= p;
  
  drawUpperPoints(points.get(5), randomtrans, start, end, books2011);
  start -= p;
  end -= p;
  
  drawUpperPoints(points.get(6), randomtrans, -180, end, books2012);
  start = 130;
  end = 180;
  
  drawLowerPoints(points.get(7), randomtrans, start, end, books2013);
  start -= p;
  end -= p;
  
  drawLowerPoints(points.get(8), randomtrans, start, end, books2014);
  start -= p;
  end -= p;
  
  drawLowerPoints(points.get(9), randomtrans, start, end, books2015);
  start -= p;
  end -= p;
  
  drawLowerPoints(points.get(10), randomtrans, start, end, books2016);
  start -= p;
  end -= p;
  
  drawLowerPoints(points.get(11), randomtrans, start, end, books2017);
  start -= p;
  end -= p;
  
  drawLowerPoints(points.get(12), randomtrans, start, end, books2018);
  start -= p;
  end -= p;
  
  drawLowerPoints(points.get(13), randomtrans, -180, end, books2019);
  
  for(int i=0; i<14; ++i)
  {
    pointstoSphere(points.get(i), randomtrans, coords.get(i), book.get(i));
  }
}

void draw() {
  background(0);
  //drawArcs();
  if(rotation)
  {
  rotateZ(2 * radians(t += (TWO_PI / 365)));
  }
  
  // earth
  noFill();
  noStroke();
  sphere(r);
  
  // points
  strokeWeight(1.5);
  cam.beginHUD();
  drawInfo();
  cp5.draw();
  cam.endHUD();
  
  for(int i=0; i<14; ++i)
  {
    convertToSpherical(points.get(i), randomtrans, book.get(i), coords.get(i));
  }
  
  int start = 0;
  int end = 99;
  deweyLines(points.get(0), start, end, coords.get(0), off[0]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(1), start, end, coords.get(1), off[1]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(2), start, end, coords.get(2), off[2]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(3), start, end, coords.get(3), off[3]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(4), start, end, coords.get(4), off[4]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(5), start, end, coords.get(5), off[5]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(6), start, end, coords.get(6), off[6]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(7), start, end, coords.get(7), off[7]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(8), start, end, coords.get(8), off[8]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(9), start, end, coords.get(9), off[9]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(10), start, end, coords.get(10), off[10]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(11), start, end, coords.get(11), off[11]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(12), start, end, coords.get(12), off[12]);
  
  start = 0;
  end = 99;
  deweyLines(points.get(13), start, end, coords.get(13), off[13]);
  
  
}
